"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Base = void 0;
const axios_1 = require("axios");
class Base {
    constructor(config) {
        /**
         * 默认内存存储
         * ttl 秒后过期
         */
        this.storage = {
            cache: Object,
            set(k, v, ttl) {
                this.cache[k] = { value: v, expiresAt: +new Date() + 1000 * ttl };
            },
            get(k) {
                return this.cache[k] && this.cache[k].expiresAt > +new Date()
                    ? this.cache[k].value
                    : null;
            }
        };
        this.config = config;
        if (config.storage) {
            this.storage.set = config.storage.set;
            this.storage.get = config.storage.get;
        }
        this.http = axios_1.default.create({ baseURL: "https://api.weixin.qq.com/" });
        this.http.interceptors.request.use((config) => __awaiter(this, void 0, void 0, function* () {
            const token = yield this.getAccessToken();
            if (token) {
                config.params = Object.assign({
                    access_token: token
                }, config.params);
            }
            return config;
        }), error => {
            return Promise.reject(error);
        });
        this.http.interceptors.response.use((resp) => __awaiter(this, void 0, void 0, function* () {
            if (resp.headers["content-type"].startsWith("application/json")) {
                const data = (resp.data instanceof Buffer
                    ? JSON.parse(resp.data.toString())
                    : resp.data);
                if (data.errcode === 40001) {
                    console.warn("Access Token 异常：", data);
                    // Access Token 无效重新获取刷新
                    this.getAccessToken(true);
                }
            }
            return resp;
        }), error => {
            return Promise.reject(error);
        });
    }
    getAccessToken(refresh = false) {
        return __awaiter(this, void 0, void 0, function* () {
            const key = `appid:${this.config.appId}:access-token`;
            let token = yield this.storage.get(key);
            if (!token || refresh) {
                if (this.config.preventTokenRefreshing) {
                    throw new Error("Trying to refresh access token in preventTokenRefreshing Mode.");
                }
                const AccessToken = yield this.getAccessTokenFromServer();
                token = AccessToken.accessToken;
                this.storage.set(key, token, AccessToken.expiresIn - 60 * 30);
            }
            return token;
        });
    }
    /**
     * ===============================================
     * 以下调用微信API
     * ===============================================
     */
    /**
     * 直接从微信服务器获取 Token
     * https://mp.weixin.qq.com/wiki?t=resource/res_main&id=mp1421140183
     * https://developers.weixin.qq.com/miniprogram/dev/api-backend/open-api/access-token/auth.getAccessToken.html
     */
    getAccessTokenFromServer() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.config.getAccessTokenFromServer) {
                return yield this.config.getAccessTokenFromServer();
            }
            const url = "https://api.weixin.qq.com/cgi-bin/token";
            const params = {
                appid: this.config.appId,
                secret: this.config.secret,
                grant_type: "client_credential"
            };
            const resp = yield axios_1.default.get(url, {
                params
            });
            if (resp.data.errcode) {
                throw new Error(`获取APP（${this.config.appId}）的AccessToken失败: ${JSON.stringify(resp.data)}`);
            }
            return {
                accessToken: resp.data.access_token,
                expiresIn: resp.data.expires_in
            };
        });
    }
}
exports.Base = Base;
