"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.User = void 0;
/**
 * 用户管理
 */
class User {
    constructor(oa) {
        this.oa = oa;
    }
    /**
     * 获取用户列表
     * https://developers.weixin.qq.com/doc/offiaccount/User_Management/Get_users_basic_information_UnionID.html#UinonId
     */
    list(next_openid) {
        return __awaiter(this, void 0, void 0, function* () {
            const url = "cgi-bin/user/get";
            const params = { next_openid };
            const resp = yield this.oa.http.get(url, { params });
            if (resp.data.errcode) {
                throw new Error(`获取用户列表失败: ${JSON.stringify(resp.data)}`);
            }
            return resp.data;
        });
    }
    /**
     * 通过 Open ID 获取用户基本信息(UnionID机制)
     * https://developers.weixin.qq.com/doc/offiaccount/User_Management/Get_users_basic_information_UnionID.html#UinonId
     */
    get({ openid, lang = "zh_CN" }) {
        return __awaiter(this, void 0, void 0, function* () {
            const url = "cgi-bin/user/info";
            const params = {
                openid,
                lang
            };
            const resp = yield this.oa.http.get(url, { params });
            if (resp.data.errcode) {
                throw new Error(`获取用户基本信息失败: ${JSON.stringify(resp.data)}`);
            }
            return resp.data;
        });
    }
    /**
     * 批量获取用户基本信息
     * https://developers.weixin.qq.com/doc/offiaccount/User_Management/Get_users_basic_information_UnionID.html#UinonId
     */
    batchget({ openids, lang = "zh_CN" }) {
        return __awaiter(this, void 0, void 0, function* () {
            const url = "cgi-bin/user/info/batchget";
            const body = {
                user_list: openids.map(e => ({ openid: e, lang }))
            };
            const resp = yield this.oa.http.post(url, body);
            if (resp.data.errcode) {
                throw new Error(`批量获取用户基本信息失败: ${JSON.stringify(resp.data)}`);
            }
            return resp.data.user_info_list;
        });
    }
    /**
     * 设置用户备注名
     * https://developers.weixin.qq.com/doc/offiaccount/User_Management/Configuring_user_notes.html
     */
    remark({ openid, remark }) {
        return __awaiter(this, void 0, void 0, function* () {
            const url = "cgi-bin/user/info/updateremark";
            const body = { openid, remark };
            const resp = yield this.oa.http.post(url, body);
            if (resp.data.errcode) {
                throw new Error(`设置用户备注名失败: ${JSON.stringify(resp.data)}`);
            }
            return resp.data.errmsg;
        });
    }
    /**
     * 获取公众号的黑名单列表
     * https://developers.weixin.qq.com/doc/offiaccount/User_Management/Manage_blacklist.html
     */
    blacklist(begin_openid) {
        return __awaiter(this, void 0, void 0, function* () {
            const url = "cgi-bin/tags/members/getblacklist";
            const body = { begin_openid };
            const resp = yield this.oa.http.post(url, body);
            if (resp.data.errcode) {
                throw new Error(`获取公众号的黑名单列表失败: ${JSON.stringify(resp.data)}`);
            }
            return resp.data;
        });
    }
    /**
     * 拉黑用户
     * https://developers.weixin.qq.com/doc/offiaccount/User_Management/Manage_blacklist.html
     */
    block(openid_list) {
        return __awaiter(this, void 0, void 0, function* () {
            const url = "cgi-bin/tags/members/batchblacklist";
            const body = { openid_list };
            const resp = yield this.oa.http.post(url, body);
            if (resp.data.errcode) {
                throw new Error(`拉黑用户失败: ${JSON.stringify(resp.data)}`);
            }
            return resp.data.errmsg;
        });
    }
    /**
     * 取消拉黑用户
     * https://developers.weixin.qq.com/doc/offiaccount/User_Management/Manage_blacklist.html
     */
    unblock(openid_list) {
        return __awaiter(this, void 0, void 0, function* () {
            const url = "cgi-bin/tags/members/batchunblacklist";
            const body = { openid_list };
            const resp = yield this.oa.http.post(url, body);
            if (resp.data.errcode) {
                throw new Error(`取消拉黑用户失败: ${JSON.stringify(resp.data)}`);
            }
            return resp.data.errmsg;
        });
    }
    /**
     * 创建标签
     * https://developers.weixin.qq.com/doc/offiaccount/User_Management/User_Tag_Management.html
     */
    createTag(name) {
        return __awaiter(this, void 0, void 0, function* () {
            const url = "cgi-bin/tags/create";
            const body = { tag: { name } };
            const resp = yield this.oa.http.post(url, body);
            if (resp.data.errcode) {
                throw new Error(`创建标签失败: ${JSON.stringify(resp.data)}`);
            }
            return resp.data.tag;
        });
    }
    /**
     * 编辑标签
     * https://developers.weixin.qq.com/doc/offiaccount/User_Management/User_Tag_Management.html
     */
    updateTag(id, name) {
        return __awaiter(this, void 0, void 0, function* () {
            const url = "cgi-bin/tags/update";
            const body = { tag: { id, name } };
            const resp = yield this.oa.http.post(url, body);
            if (resp.data.errcode) {
                throw new Error(`编辑标签失败: ${JSON.stringify(resp.data)}`);
            }
            return resp.data.errmsg;
        });
    }
    /**
     * 删除标签
     * https://developers.weixin.qq.com/doc/offiaccount/User_Management/User_Tag_Management.html
     */
    deleteTag(id) {
        return __awaiter(this, void 0, void 0, function* () {
            const url = "cgi-bin/tags/delete";
            const body = { tag: { id } };
            const resp = yield this.oa.http.post(url, body);
            if (resp.data.errcode) {
                throw new Error(`删除标签失败: ${JSON.stringify(resp.data)}`);
            }
            return resp.data.errmsg;
        });
    }
    /**
     * 获取公众号已创建的标签
     * https://developers.weixin.qq.com/doc/offiaccount/User_Management/User_Tag_Management.html
     */
    listTag() {
        return __awaiter(this, void 0, void 0, function* () {
            const url = "cgi-bin/tags/get";
            const resp = yield this.oa.http.get(url);
            if (resp.data.errcode) {
                throw new Error(`获取公众号已创建的标签失败: ${JSON.stringify(resp.data)}`);
            }
            return resp.data.tags;
        });
    }
    /**
     * 获取标签下粉丝列表
     * https://developers.weixin.qq.com/doc/offiaccount/User_Management/User_Tag_Management.html
     */
    usersOfTag(tagid, next_openid) {
        return __awaiter(this, void 0, void 0, function* () {
            const url = "cgi-bin/user/tag/get";
            const body = { tagid, next_openid };
            const resp = yield this.oa.http.post(url, body);
            if (resp.data.errcode) {
                throw new Error(`获取标签下粉丝列表失败: ${JSON.stringify(resp.data)}`);
            }
            return resp.data;
        });
    }
    /**
     * 批量为用户打标签
     * https://developers.weixin.qq.com/doc/offiaccount/User_Management/User_Tag_Management.html
     */
    tagUsers(tagid, openid_list) {
        return __awaiter(this, void 0, void 0, function* () {
            const url = "cgi-bin/tags/members/batchtagging";
            const body = { tagid, openid_list };
            const resp = yield this.oa.http.post(url, body);
            if (resp.data.errcode) {
                throw new Error(`批量为用户打标签失败: ${JSON.stringify(resp.data)}`);
            }
            return resp.data.errmsg;
        });
    }
    /**
     * 批量为用户取消标签
     * https://developers.weixin.qq.com/doc/offiaccount/User_Management/User_Tag_Management.html
     */
    untagUsers(tagid, openid_list) {
        return __awaiter(this, void 0, void 0, function* () {
            const url = "cgi-bin/tags/members/batchuntagging";
            const body = { tagid, openid_list };
            const resp = yield this.oa.http.post(url, body);
            if (resp.data.errcode) {
                throw new Error(`批量为用户取消标签失败: ${JSON.stringify(resp.data)}`);
            }
            return resp.data.errmsg;
        });
    }
    /**
     * 获取用户身上的标签列表
     * https://developers.weixin.qq.com/doc/offiaccount/User_Management/User_Tag_Management.html
     */
    userTags(openid) {
        return __awaiter(this, void 0, void 0, function* () {
            const url = "cgi-bin/tags/getidlist";
            const body = { openid };
            const resp = yield this.oa.http.post(url, body);
            if (resp.data.errcode) {
                throw new Error(`获取用户身上的标签列表失败: ${JSON.stringify(resp.data)}`);
            }
            return resp.data.tagid_list;
        });
    }
}
exports.User = User;
